@@ -135,6 +135,19 @@ fn sort() {
     v.sort();
 }
 
+fn swap() {
+    let mut v = vec![1, 2, 3, 4];
+    v.swap(2, 2);
+}
+
+fn swap_remove() {
+    let mut a = 0;
+    let mut b = 1;
+    let mut vec = vec![&mut a, &mut b];
+
+    vec.swap_remove(1);
+}
+
 fn main() {
     assert_eq!(vec_reallocate().len(), 5);
 
@@ -161,4 +174,6 @@ fn main() {
     push_str_ptr_stable();
 
     sort();
+    swap();
+    swap_remove();
 }
