@@ -1,38 +1,83 @@
 //! Completion of paths, i.e. `some::prefix::$0`.
 
-use hir::HasVisibility;
-use rustc_hash::FxHashSet;
-use syntax::AstNode;
+use std::iter;
 
-use crate::{CompletionContext, Completions};
+use rustc_hash::FxHashSet;
+use syntax::{ast, AstNode};
+
+use crate::{
+    context::PathCompletionContext, patterns::ImmediateLocation, CompletionContext, Completions,
+};
 
 pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionContext) {
-    if ctx.is_path_disallowed() || ctx.expects_item() {
+    if ctx.is_path_disallowed() || ctx.has_impl_or_trait_prev_sibling() {
         return;
     }
-    let path = match &ctx.path_qual {
-        Some(path) => path.clone(),
-        None => return,
+    let (path, use_tree_parent) = match &ctx.path_context {
+        Some(PathCompletionContext { qualifier: Some(qualifier), use_tree_parent, .. }) => {
+            (qualifier, *use_tree_parent)
+        }
+        _ => return,
     };
 
-    let resolution = match ctx.sema.resolve_path(&path) {
+    let resolution = match ctx.sema.resolve_path(path) {
         Some(res) => res,
         None => return,
     };
+
     let context_module = ctx.scope.module();
-    if ctx.expects_assoc_item() {
-        if let hir::PathResolution::Def(hir::ModuleDef::Module(module)) = resolution {
-            let module_scope = module.scope(ctx.db, context_module);
-            for (name, def) in module_scope {
-                if let hir::ScopeDef::MacroDef(macro_def) = def {
-                    acc.add_macro(ctx, Some(name.clone()), macro_def);
-                }
-                if let hir::ScopeDef::ModuleDef(hir::ModuleDef::Module(_)) = def {
-                    acc.add_resolution(ctx, name, &def);
+
+    match ctx.completion_location {
+        Some(ImmediateLocation::ItemList | ImmediateLocation::Trait | ImmediateLocation::Impl) => {
+            if let hir::PathResolution::Def(hir::ModuleDef::Module(module)) = resolution {
+                for (name, def) in module.scope(ctx.db, context_module) {
+                    if let hir::ScopeDef::MacroDef(macro_def) = def {
+                        if macro_def.is_fn_like() {
+                            acc.add_macro(ctx, Some(name.clone()), macro_def);
+                        }
+                    }
+                    if let hir::ScopeDef::ModuleDef(hir::ModuleDef::Module(_)) = def {
+                        acc.add_resolution(ctx, name, &def);
+                    }
                 }
             }
+            return;
+        }
+        Some(ImmediateLocation::Visibility(_)) => {
+            if let hir::PathResolution::Def(hir::ModuleDef::Module(resolved)) = resolution {
+                if let Some(current_module) = ctx.scope.module() {
+                    if let Some(next) = current_module
+                        .path_to_root(ctx.db)
+                        .into_iter()
+                        .take_while(|&it| it != resolved)
+                        .next()
+                    {
+                        if let Some(name) = next.name(ctx.db) {
+                            acc.add_resolution(ctx, name, &hir::ScopeDef::ModuleDef(next.into()));
+                        }
+                    }
+                }
+            }
+            return;
+        }
+        _ => (),
+    }
+
+    if ctx.in_use_tree() {
+        if iter::successors(Some(path.clone()), |p| p.qualifier())
+            .all(|p| p.segment().and_then(|s| s.super_token()).is_some())
+        {
+            acc.add_keyword(ctx, "super::");
+        }
+        // only show `self` in a new use-tree when the qualifier doesn't end in self
+        if use_tree_parent
+            && !matches!(
+                path.segment().and_then(|it| it.kind()),
+                Some(ast::PathSegmentKind::SelfKw)
+            )
+        {
+            acc.add_keyword(ctx, "self");
         }
-        return;
     }
 
     // Add associated types on type parameters and `Self`.
@@ -45,10 +90,10 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
         hir::PathResolution::Def(hir::ModuleDef::Module(module)) => {
             let module_scope = module.scope(ctx.db, context_module);
             for (name, def) in module_scope {
-                if ctx.use_item_syntax.is_some() {
+                if ctx.in_use_tree() {
                     if let hir::ScopeDef::Unknown = def {
-                        if let Some(name_ref) = ctx.name_ref_syntax.as_ref() {
-                            if name_ref.syntax().text() == name.to_string().as_str() {
+                        if let Some(ast::NameLike::NameRef(name_ref)) = ctx.name_syntax.as_ref() {
+                            if name_ref.syntax().text() == name.to_smol_str().as_str() {
                                 // for `use self::foo$0`, don't suggest `foo` as a completion
                                 cov_mark::hit!(dont_complete_current_use);
                                 continue;
@@ -57,14 +102,37 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
                     }
                 }
 
-                acc.add_resolution(ctx, name, &def);
+                let add_resolution = match def {
+                    // Don't suggest attribute macros and derives.
+                    hir::ScopeDef::MacroDef(mac) => mac.is_fn_like(),
+                    // no values in type places
+                    hir::ScopeDef::ModuleDef(
+                        hir::ModuleDef::Function(_)
+                        | hir::ModuleDef::Variant(_)
+                        | hir::ModuleDef::Static(_),
+                    )
+                    | hir::ScopeDef::Local(_) => !ctx.expects_type(),
+                    // unless its a constant in a generic arg list position
+                    hir::ScopeDef::ModuleDef(hir::ModuleDef::Const(_)) => {
+                        !ctx.expects_type() || ctx.expects_generic_arg()
+                    }
+                    _ => true,
+                };
+
+                if add_resolution {
+                    acc.add_resolution(ctx, name, &def);
+                }
             }
         }
-        hir::PathResolution::Def(def @ hir::ModuleDef::Adt(_))
-        | hir::PathResolution::Def(def @ hir::ModuleDef::TypeAlias(_))
-        | hir::PathResolution::Def(def @ hir::ModuleDef::BuiltinType(_)) => {
+        hir::PathResolution::Def(
+            def
+            @
+            (hir::ModuleDef::Adt(_)
+            | hir::ModuleDef::TypeAlias(_)
+            | hir::ModuleDef::BuiltinType(_)),
+        ) => {
             if let hir::ModuleDef::Adt(hir::Adt::Enum(e)) = def {
-                add_enum_variants(ctx, acc, e);
+                add_enum_variants(acc, ctx, e);
             }
             let ty = match def {
                 hir::ModuleDef::Adt(adt) => adt.ty(ctx.db),
@@ -72,7 +140,7 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
                     let ty = a.ty(ctx.db);
                     if let Some(hir::Adt::Enum(e)) = ty.as_adt() {
                         cov_mark::hit!(completes_variant_through_alias);
-                        add_enum_variants(ctx, acc, e);
+                        add_enum_variants(acc, ctx, e);
                     }
                     ty
                 }
@@ -94,22 +162,12 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
             if let Some(krate) = krate {
                 let traits_in_scope = ctx.scope.traits_in_scope();
                 ty.iterate_path_candidates(ctx.db, krate, &traits_in_scope, None, |_ty, item| {
-                    if context_module.map_or(false, |m| !item.is_visible_from(ctx.db, m)) {
-                        return None;
-                    }
-                    match item {
-                        hir::AssocItem::Function(func) => acc.add_function(ctx, func, None),
-                        hir::AssocItem::Const(ct) => acc.add_const(ctx, ct),
-                        hir::AssocItem::TypeAlias(ty) => acc.add_type_alias(ctx, ty),
-                    }
+                    add_assoc_item(acc, ctx, item);
                     None::<()>
                 });
 
                 // Iterate assoc types separately
                 ty.iterate_assoc_items(ctx.db, krate, |item| {
-                    if context_module.map_or(false, |m| !item.is_visible_from(ctx.db, m)) {
-                        return None;
-                    }
                     if let hir::AssocItem::TypeAlias(ty) = item {
                         acc.add_type_alias(ctx, ty)
                     }
@@ -120,14 +178,7 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
         hir::PathResolution::Def(hir::ModuleDef::Trait(t)) => {
             // Handles `Trait::assoc` as well as `<Ty as Trait>::assoc`.
             for item in t.items(ctx.db) {
-                if context_module.map_or(false, |m| !item.is_visible_from(ctx.db, m)) {
-                    continue;
-                }
-                match item {
-                    hir::AssocItem::Function(func) => acc.add_function(ctx, func, None),
-                    hir::AssocItem::Const(ct) => acc.add_const(ctx, ct),
-                    hir::AssocItem::TypeAlias(ty) => acc.add_type_alias(ctx, ty),
-                }
+                add_assoc_item(acc, ctx, item);
             }
         }
         hir::PathResolution::TypeParam(_) | hir::PathResolution::SelfType(_) => {
@@ -139,293 +190,78 @@ pub(crate) fn complete_qualified_path(acc: &mut Completions, ctx: &CompletionCon
                 };
 
                 if let Some(hir::Adt::Enum(e)) = ty.as_adt() {
-                    add_enum_variants(ctx, acc, e);
+                    add_enum_variants(acc, ctx, e);
                 }
 
                 let traits_in_scope = ctx.scope.traits_in_scope();
                 let mut seen = FxHashSet::default();
                 ty.iterate_path_candidates(ctx.db, krate, &traits_in_scope, None, |_ty, item| {
-                    if context_module.map_or(false, |m| !item.is_visible_from(ctx.db, m)) {
-                        return None;
-                    }
-
                     // We might iterate candidates of a trait multiple times here, so deduplicate
                     // them.
                     if seen.insert(item) {
-                        match item {
-                            hir::AssocItem::Function(func) => acc.add_function(ctx, func, None),
-                            hir::AssocItem::Const(ct) => acc.add_const(ctx, ct),
-                            hir::AssocItem::TypeAlias(ty) => acc.add_type_alias(ctx, ty),
-                        }
+                        add_assoc_item(acc, ctx, item);
                     }
                     None::<()>
                 });
             }
         }
+        hir::PathResolution::Macro(mac) => acc.add_macro(ctx, None, mac),
         _ => {}
     }
 }
 
-fn add_enum_variants(ctx: &CompletionContext, acc: &mut Completions, e: hir::Enum) {
-    for variant in e.variants(ctx.db) {
-        acc.add_enum_variant(ctx, variant, None);
+fn add_assoc_item(acc: &mut Completions, ctx: &CompletionContext, item: hir::AssocItem) {
+    match item {
+        hir::AssocItem::Function(func) if !ctx.expects_type() => acc.add_function(ctx, func, None),
+        hir::AssocItem::Const(ct) if !ctx.expects_type() || ctx.expects_generic_arg() => {
+            acc.add_const(ctx, ct)
+        }
+        hir::AssocItem::TypeAlias(ty) => acc.add_type_alias(ctx, ty),
+        _ => (),
     }
 }
 
+fn add_enum_variants(acc: &mut Completions, ctx: &CompletionContext, e: hir::Enum) {
+    if ctx.expects_type() {
+        return;
+    }
+    e.variants(ctx.db).into_iter().for_each(|variant| acc.add_enum_variant(ctx, variant, None));
+}
+
 #[cfg(test)]
 mod tests {
     use expect_test::{expect, Expect};
 
-    use crate::{
-        test_utils::{check_edit, completion_list},
-        CompletionKind,
-    };
+    use crate::tests::{check_edit, completion_list_no_kw};
 
     fn check(ra_fixture: &str, expect: Expect) {
-        let actual = completion_list(ra_fixture, CompletionKind::Reference);
+        let actual = completion_list_no_kw(ra_fixture);
         expect.assert_eq(&actual);
     }
 
-    fn check_builtin(ra_fixture: &str, expect: Expect) {
-        let actual = completion_list(ra_fixture, CompletionKind::BuiltinType);
-        expect.assert_eq(&actual);
-    }
-
-    #[test]
-    fn dont_complete_current_use() {
-        cov_mark::check!(dont_complete_current_use);
-        check(r#"use self::foo$0;"#, expect![[""]]);
-    }
-
-    #[test]
-    fn dont_complete_current_use_in_braces_with_glob() {
-        check(
-            r#"
-mod foo { pub struct S; }
-use self::{foo::*, bar$0};
-"#,
-            expect![[r#"
-                st S
-                md foo
-            "#]],
-        );
-    }
-
-    #[test]
-    fn dont_complete_primitive_in_use() {
-        check_builtin(r#"use self::$0;"#, expect![[""]]);
-    }
-
-    #[test]
-    fn dont_complete_primitive_in_module_scope() {
-        check_builtin(r#"fn foo() { self::$0 }"#, expect![[""]]);
-    }
-
-    #[test]
-    fn completes_primitives() {
-        check_builtin(
-            r#"fn main() { let _: $0 = 92; }"#,
-            expect![[r#"
-                bt u32
-                bt bool
-                bt u8
-                bt isize
-                bt u16
-                bt u64
-                bt u128
-                bt f32
-                bt i128
-                bt i16
-                bt str
-                bt i64
-                bt char
-                bt f64
-                bt i32
-                bt i8
-                bt usize
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_mod_with_same_name_as_function() {
-        check(
-            r#"
-use self::my::$0;
-
-mod my { pub struct Bar; }
-fn my() {}
-"#,
-            expect![[r#"
-                st Bar
-            "#]],
-        );
-    }
-
-    #[test]
-    fn filters_visibility() {
-        check(
-            r#"
-use self::my::$0;
-
-mod my {
-    struct Bar;
-    pub struct Foo;
-    pub use Bar as PublicBar;
-}
-"#,
-            expect![[r#"
-                st Foo
-                st PublicBar
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_use_item_starting_with_self() {
-        check(
-            r#"
-use self::m::$0;
-
-mod m { pub struct Bar; }
-"#,
-            expect![[r#"
-                st Bar
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_use_item_starting_with_crate() {
-        check(
-            r#"
-//- /lib.rs
-mod foo;
-struct Spam;
-//- /foo.rs
-use crate::Sp$0
-"#,
-            expect![[r#"
-                md foo
-                st Spam
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_nested_use_tree() {
-        check(
-            r#"
-//- /lib.rs
-mod foo;
-struct Spam;
-//- /foo.rs
-use crate::{Sp$0};
-"#,
-            expect![[r#"
-                md foo
-                st Spam
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_deeply_nested_use_tree() {
-        check(
-            r#"
-//- /lib.rs
-mod foo;
-pub mod bar {
-    pub mod baz {
-        pub struct Spam;
-    }
-}
-//- /foo.rs
-use crate::{bar::{baz::Sp$0}};
-"#,
-            expect![[r#"
-                st Spam
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_enum_variant() {
-        check(
-            r#"
-enum E { Foo, Bar(i32) }
-fn foo() { let _ = E::$0 }
-"#,
-            expect![[r#"
-                ev Foo    ()
-                ev Bar(…) (i32)
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_struct_associated_items() {
-        check(
-            r#"
-//- /lib.rs
-struct S;
-
-impl S {
-    fn a() {}
-    fn b(&self) {}
-    const C: i32 = 42;
-    type T = i32;
-}
-
-fn foo() { let _ = S::$0 }
-"#,
-            expect![[r#"
-                fn a()  fn()
-                me b(…) fn(&self)
-                ct C    const C: i32 = 42;
-                ta T    type T = i32;
-            "#]],
-        );
-    }
-
     #[test]
     fn associated_item_visibility() {
         check(
             r#"
-struct S;
+//- /lib.rs crate:lib new_source_root:library
+pub struct S;
 
-mod m {
-    impl super::S {
-        pub(crate) fn public_method() { }
-        fn private_method() { }
-        pub(crate) type PublicType = u32;
-        type PrivateType = u32;
-        pub(crate) const PUBLIC_CONST: u32 = 1;
-        const PRIVATE_CONST: u32 = 1;
-    }
+impl S {
+    pub fn public_method() { }
+    fn private_method() { }
+    pub type PublicType = u32;
+    type PrivateType = u32;
+    pub const PUBLIC_CONST: u32 = 1;
+    const PRIVATE_CONST: u32 = 1;
 }
 
-fn foo() { let _ = S::$0 }
+//- /main.rs crate:main deps:lib new_source_root:local
+fn foo() { let _ = lib::S::$0 }
 "#,
             expect![[r#"
                 fn public_method() fn()
-                ct PUBLIC_CONST    pub(crate) const PUBLIC_CONST: u32 = 1;
-                ta PublicType      pub(crate) type PublicType = u32;
-            "#]],
-        );
-    }
-
-    #[test]
-    fn completes_enum_associated_method() {
-        check(
-            r#"
-enum E {};
-impl E { fn m() { } }
-
-fn foo() { let _ = E::$0 }
-        "#,
-            expect![[r#"
-                fn m() fn()
+                ct PUBLIC_CONST    pub const PUBLIC_CONST: u32 = 1;
+                ta PublicType      pub type PublicType = u32;
             "#]],
         );
     }
@@ -445,22 +281,6 @@ fn foo() { let _ = U::$0 }
         );
     }
 
-    #[test]
-    fn completes_use_paths_across_crates() {
-        check(
-            r#"
-//- /main.rs crate:main deps:foo
-use foo::$0;
-
-//- /foo/lib.rs crate:foo
-pub mod bar { pub struct S; }
-"#,
-            expect![[r#"
-                md bar
-            "#]],
-        );
-    }
-
     #[test]
     fn completes_trait_associated_method_1() {
         check(
@@ -470,7 +290,7 @@ trait Trait { fn m(); }
 fn foo() { let _ = Trait::$0 }
 "#,
             expect![[r#"
-                fn m() fn()
+                fn m() (as Trait) fn()
             "#]],
         );
     }
@@ -487,7 +307,7 @@ impl Trait for S {}
 fn foo() { let _ = S::$0 }
 "#,
             expect![[r#"
-                fn m() fn()
+                fn m() (as Trait) fn()
             "#]],
         );
     }
@@ -504,7 +324,7 @@ impl Trait for S {}
 fn foo() { let _ = <S as Trait>::$0 }
 "#,
             expect![[r#"
-                fn m() fn()
+                fn m() (as Trait) fn()
             "#]],
         );
     }
@@ -530,14 +350,14 @@ trait Sub: Super {
 fn foo<T: Sub>() { T::$0 }
 "#,
             expect![[r#"
-                ta SubTy        type SubTy;
-                ta Ty           type Ty;
-                ct C2           const C2: ();
-                fn subfunc()    fn()
-                me submethod(…) fn(&self)
-                ct CONST        const CONST: u8;
-                fn func()       fn()
-                me method(…)    fn(&self)
+                ta SubTy (as Sub)        type SubTy;
+                ta Ty (as Super)         type Ty;
+                ct C2 (as Sub)           const C2: ();
+                fn subfunc() (as Sub)    fn()
+                me submethod(…) (as Sub) fn(&self)
+                ct CONST (as Super)      const CONST: u8;
+                fn func() (as Super)     fn()
+                me method(…) (as Super)  fn(&self)
             "#]],
         );
     }
@@ -570,14 +390,14 @@ impl<T> Sub for Wrap<T> {
 }
 "#,
             expect![[r#"
-                ta SubTy        type SubTy;
-                ta Ty           type Ty;
-                ct CONST        const CONST: u8 = 0;
-                fn func()       fn()
-                me method(…)    fn(&self)
-                ct C2           const C2: () = ();
-                fn subfunc()    fn()
-                me submethod(…) fn(&self)
+                ta SubTy (as Sub)        type SubTy;
+                ta Ty (as Super)         type Ty;
+                ct CONST (as Super)      const CONST: u8 = 0;
+                fn func() (as Super)     fn()
+                me method(…) (as Super)  fn(&self)
+                ct C2 (as Sub)           const C2: () = ();
+                fn subfunc() (as Sub)    fn()
+                me submethod(…) (as Sub) fn(&self)
             "#]],
         );
     }
@@ -617,59 +437,28 @@ fn main() { let _ = crate::$0 }
     }
 
     #[test]
-    fn completes_in_assoc_item_list() {
+    fn does_not_complete_non_fn_macros() {
         check(
             r#"
-#[macro_export]
-macro_rules! foo { () => {} }
-mod bar {}
-
-struct MyStruct {}
-impl MyStruct {
-    crate::$0
+mod m {
+    #[rustc_builtin_macro]
+    pub macro Clone {}
 }
+
+fn f() {m::$0}
 "#,
-            expect![[r##"
-                md bar
-                ma foo! #[macro_export] macro_rules! foo
-            "##]],
+            expect![[r#""#]],
         );
-    }
-
-    #[test]
-    #[ignore] // FIXME doesn't complete anything atm
-    fn completes_in_item_list() {
         check(
             r#"
-struct MyStruct {}
-macro_rules! foo {}
-mod bar {}
-
-crate::$0
-"#,
-            expect![[r#"
-                md bar
-                ma foo! macro_rules! foo
-            "#]],
-        )
-    }
-
-    #[test]
-    fn test_super_super_completion() {
-        check(
-            r#"
-mod a {
-    const A: usize = 0;
-    mod b {
-        const B: usize = 0;
-        mod c { use super::super::$0 }
-    }
+mod m {
+    #[rustc_builtin_macro]
+    pub macro bench {}
 }
+
+fn f() {m::$0}
 "#,
-            expect![[r#"
-                md b
-                ct A
-            "#]],
+            expect![[r#""#]],
         );
     }
 
@@ -861,4 +650,34 @@ fn main() {
             "#]],
         );
     }
+
+    #[test]
+    fn respects_doc_hidden() {
+        cov_mark::check!(qualified_path_doc_hidden);
+        check(
+            r#"
+//- /lib.rs crate:lib deps:dep
+fn f() {
+    dep::$0
+}
+
+//- /dep.rs crate:dep
+#[doc(hidden)]
+#[macro_export]
+macro_rules! m {
+    () => {}
+}
+
+#[doc(hidden)]
+pub fn f() {}
+
+#[doc(hidden)]
+pub struct S;
+
+#[doc(hidden)]
+pub mod m {}
+            "#,
+            expect![[r#""#]],
+        )
+    }
 }
